@@ -31,17 +31,7 @@
  * One service known to work with obsrinex output is [CSRS-PPP]:
  *  https://webapp.geod.nrcan.gc.ca/geod/tools-outils/ppp.php
  *
- * Examples:
- *    To collect 4 hours of samples as 30 second intervals:
- *        # gpsrinex -i 30 -n 480
- *
- * To generate RINEX 3 from a u-blox capture file:
- *     Grab 4 hours of raw live data:
- *         # gpspipe -x 14400 -R > 4h-raw.ubx
- *     Feed that data to gpsfake:
- *         # gpsfake -1 -P 3000 4h-raw.ubx
- *     In another window, convert that raw to RINEX 3:
- *         # gpsrinex -i 1 -n 1000000
+ * See the gpsrinex man page for usage examples.
  *
  * See also:
  *     [1] RINEX: The Receiver Independent Exchange Format, Version 3.04
@@ -56,7 +46,7 @@
  *     [4] RTKLIB: An Open Source Program Package for GNSS Positioning
  *     http://www.rtklib.com/
  *
- * This file is Copyright 2018 by the GPSD project
+ * This file is Copyright by the GPSD project
  * SPDX-License-Identifier: BSD-2-clause
  *
  */
@@ -528,9 +518,12 @@ static void print_rinex_header(void)
     (void)fprintf(log_file, "%10.3f%50s%-20s\n",
                   (double)sample_interval_ms / 1000.0, "", "INTERVAL");
 
-    // GPS time not UTC
+    /* GPS time not UTC.  The data and time fields are I6, but some
+     * PPP services still want a leading zero.  So "    02", not "     2".
+     */
     first_time = gmtime_r(&(first_mtime.tv_sec), &tm_buf);
-    (void)fprintf(log_file, "%6d%6d%6d%6d%6d%5d.%07ld%8s%9s%-20s\n",
+    (void)fprintf(log_file, "%6d    %02d    %02d    %02d    %02d"
+                            "   %02d.%07ld%8s%9s%-20s\n",
          first_time->tm_year + 1900,
          first_time->tm_mon + 1,
          first_time->tm_mday,
@@ -541,9 +534,12 @@ static void print_rinex_header(void)
          "GPS", "",
          "TIME OF FIRST OBS");
 
-    // GPS time not UTC
+    /* GPS time not UTC.  The data and time fields are I6, but some
+     * PPP services still want a leading zero.  So "    02", not "     2".
+     */
     last_time = gmtime_r(&(last_mtime.tv_sec), &tm_buf);
-    (void)fprintf(log_file, "%6d%6d%6d%6d%6d%5d.%07ld%8s%9s%-20s\n",
+    (void)fprintf(log_file, "%6d    %02d    %02d    %02d    %02d"
+                            "   %02d.%07ld%8s%9s%-20s\n",
          last_time->tm_year + 1900,
          last_time->tm_mon + 1,
          last_time->tm_mday,
@@ -898,8 +894,8 @@ static void print_raw(struct gps_data_t *gpsdata)
             continue;
         }
         // prevent separate sigid from double counting gnssid:svid
-        if ((last_gnssid == gpsdata->raw.meas[i].gnssid) &&
-            (last_svid == gpsdata->raw.meas[i].svid)) {
+        if (last_gnssid == gpsdata->raw.meas[i].gnssid &&
+            last_svid == gpsdata->raw.meas[i].svid) {
             // duplicate sat
             continue;
         }
@@ -937,13 +933,9 @@ static void print_raw(struct gps_data_t *gpsdata)
      * then later they can be ouput it arbitrary orders  */
     memset(obs_items, 0, sizeof(obs_items));
     for (i = 0; i < nrec; i++) {
-        unsigned char gnssid;
-        unsigned char svid;
-        unsigned char sigid;
-
-        gnssid = gpsdata->raw.meas[i].gnssid;
-        svid = gpsdata->raw.meas[i].svid;
-        sigid = gpsdata->raw.meas[i].sigid;
+        const unsigned char gnssid = gpsdata->raw.meas[i].gnssid;
+        const unsigned char svid = gpsdata->raw.meas[i].svid;
+        const unsigned char sigid = gpsdata->raw.meas[i].sigid;
 
         if (DEBUG_RAW <= debug) {
             (void)fprintf(stderr,"record: %u:%u:%u %s\n",
@@ -951,14 +943,14 @@ static void print_raw(struct gps_data_t *gpsdata)
                           gpsdata->raw.meas[i].obs_code);
         }
 
-        if (0 == gpsdata->raw.meas[i].svid) {
+        if (0 == svid) {
             // should not happen...
             continue;
         }
 
         // line can be longer than 80 chars in RINEX 3
-        if ((last_gnssid != gpsdata->raw.meas[i].gnssid) ||
-            (last_svid != gpsdata->raw.meas[i].svid)) {
+        if (last_gnssid != gnssid ||
+            last_svid != svid) {
             char rinex_gnssid;
 
             rinex_gnssid = gnssid2rinex(last_gnssid);
@@ -977,17 +969,39 @@ static void print_raw(struct gps_data_t *gpsdata)
                 }
                 (void)fputs("\n", tmp_file);
             }
-
-            // ready for new sat
-            memset(obs_items, 0, sizeof(obs_items));
         }
 
-        last_gnssid = gpsdata->raw.meas[i].gnssid;
-        last_svid = gpsdata->raw.meas[i].svid;
+        last_gnssid = gnssid;
+        last_svid = svid;
 
         one_sig(&gpsdata->raw.meas[i]);
 
     }
+
+    // Process the last onem why can't this go in the for loop above?
+    {
+        char rinex_gnssid;
+
+        rinex_gnssid = gnssid2rinex(last_gnssid);
+
+        if (0 != last_svid) {
+            int j;
+
+            (void)fprintf(tmp_file, "%c%02d", rinex_gnssid, last_svid);
+            for (j = 0; j < MAX_TYPES; j++) {
+                int obs = obs_set[last_gnssid][j];
+
+                if (CODEMAX == obs) {
+                    break;
+                }
+                (void)fprintf(tmp_file, "%16s", obs_items[obs]);
+            }
+            (void)fputs("\n", tmp_file);
+        }
+    }
+    // ready for new sat
+    memset(obs_items, 0, sizeof(obs_items));
+
     sample_count--;
 }
 
@@ -1138,6 +1152,7 @@ int main(int argc, char **argv)
     char   *file_in = NULL;
     int timeout = 10;
     double f;
+    int err;
 
     progname = argv[0];
 
@@ -1185,9 +1200,15 @@ int main(int argc, char **argv)
             gps_enable_debug(debug, log_file);
             break;
         case 'f':       // Output file name.
+            if (NULL != file_out) {
+                free(file_out);
+            }
             file_out = strdup(optarg);
             break;
         case 'F':       // input file name.
+            if (NULL != file_in) {
+                free(file_in);
+            }
             file_in = strdup(optarg);
             break;
         case 'i':               // set sampling interval
@@ -1209,6 +1230,12 @@ int main(int argc, char **argv)
         case 'V':
             (void)fprintf(stderr, "%s: version %s (revision %s)\n",
                           progname, VERSION, REVISION);
+            if (NULL != file_in) {
+                free(file_in);
+            }
+            if (NULL != file_out) {
+                free(file_out);
+            }
             exit(EXIT_SUCCESS);
         case AGENCY:
             strlcpy(agency, optarg, sizeof(agency));
@@ -1251,6 +1278,12 @@ int main(int argc, char **argv)
         case 'h':
             FALLTHROUGH
         default:
+            if (NULL != file_in) {
+                free(file_in);
+            }
+            if (NULL != file_out) {
+                free(file_out);
+            }
             usage();
             // NOTREACHED
         }
@@ -1288,15 +1321,17 @@ int main(int argc, char **argv)
     if (NULL == file_out) {
         (void)strftime(tmstr, sizeof(tmstr), "gpsrinex%Y%j%H%M%S.obs",
                        report_time);
-        file_out = tmstr;
+        file_out = strdup(tmstr);
     }
     log_file = fopen(file_out, "w");
-    if (log_file == NULL) {
+    if (NULL == log_file) {
         syslog(LOG_ERR, "ERROR: Failed to open %s: %s",
                file_out, strerror(errno));
+        free(file_out);      // pacify -Wanalyzer-malloc-leak
         exit(3);
     }
 
+    free(file_out);      // pacify -Wanalyzer-malloc-leak
     // clear the counts
     memset(obs_cnt, 0, sizeof(obs_cnt));
 
@@ -1305,10 +1340,11 @@ int main(int argc, char **argv)
     (void)signal(SIGQUIT, quit_handler);
     (void)signal(SIGINT, quit_handler);
 
-    if (0 > gps_open(source.server, source.port, &gpsdata)) {
+    err = gps_open(source.server, source.port, &gpsdata);
+    if (0 > err) {
         (void)fprintf(stderr,
-                      "%s: no gpsd running or network error: %d, %s(%d)\n",
-                      progname, errno, gps_errstr(errno), errno);
+                      "%s: gps_open() failed  %s(%d) errno %s(%d)\n",
+                      progname, gps_errstr(err), err, strerror(errno), errno);
         exit(EXIT_FAILURE);
     }
     if (NULL != source.device) {
@@ -1372,6 +1408,9 @@ int main(int argc, char **argv)
         SIGINT != sig_flag) {
         syslog(LOG_INFO, "exiting, signal %d received", sig_flag);
     }
+    if (NULL != file_in) {
+        free(file_in);      // pacify -Wanalyzer-malloc-leak
+    }
     exit(EXIT_SUCCESS);
 }
 
