@@ -218,9 +218,11 @@ static long compare_oid(const char *oid1, const char *oid2)
      * someone add a leading zero and mess things up. */
     int in1 = 0;
     int in2 = 0;
-    long part1, part2, ret = 0;
+    long ret = 0;
 
     while (1) {
+        long part1, part2;
+
         if ('.' != oid1[in1] ||
             '.' != oid2[in2]) {
             // legally, the only option is nul.
@@ -362,8 +364,6 @@ static void put_line(const char *outbuf)
 static void get_one(gps_mask_t need)
 {
     struct timespec ts_start, ts_now;
-    int status, i;
-    double snr_total = 0;
 
     if (ONLINE_SET == need) {
         // nothing needed
@@ -392,6 +392,7 @@ static void get_one(gps_mask_t need)
 
     // timout is in micro seconds.
     while (gps_waiting(&gpsdata, 2 * US_IN_SEC)) {
+        int status;
 
         // wait 3 seconds, tops.
         clock_gettime(CLOCK_REALTIME, &ts_now);
@@ -428,7 +429,10 @@ static void get_one(gps_mask_t need)
     }
     if (SATELLITE_SET == (need & gpsdata.set)) {
         // compute a derived value: snr_avg
-        for(i = 0; i <= gpsdata.satellites_used; i++) {
+        double snr_total = 0;
+        int i;
+
+        for(i = 0; i < MAXCHANNELS; i++) {
             if (0 < gpsdata.skyview[i].used &&
                 1 <  gpsdata.skyview[i].ss) {
                 // printf("i: %d, P:%d, ss: %f\n", i, gpsdata.skyview[i].PRN,
@@ -638,7 +642,7 @@ int main (int argc, char **argv)
 {
     bool persist = false;
     bool do_usage = false;
-    int status;
+    int err;
     char oid[40] = "";       // requested OID
     struct fixsource_t source;
     const struct oid_mib_xlate *pxlate;
@@ -751,10 +755,11 @@ int main (int argc, char **argv)
     }
 
     // Open the stream to gpsd
-    status = gps_open(source.server, source.port, &gpsdata);
-    if (0 != status) {
-        (void)fprintf(logfd, PROGNAME ": ERROR: connection failed: %d\n",
-                      status);
+    err = gps_open(source.server, source.port, &gpsdata);
+    if (0 != err) {
+        (void)fprintf(stderr,
+                      "gpssnmp: gps_open() failed  %s(%d) errno %s(%d)\n",
+                      gps_errstr(err), err, strerror(errno), errno);
         exit(1);
     }
     // we want JSON
