@@ -272,6 +272,9 @@ static void quit_handler(int signum)
 
 static void usage(void)
 {
+    (void)fprintf(stderr, "%s: version %s (revision %s)\n\n",
+                  progname, VERSION, REVISION);
+
     (void)fprintf(stderr,
          "Usage: %s [OPTIONS] [server[:port:[device]]]\n\n"
          "  -?                  Show this help, then exit\n"
@@ -302,12 +305,15 @@ static void usage(void)
          "  -l                  List available exports, then exit\n"
          "  -m MINMOVE          Minimum move in meters to log\n"
          "  -r                  Retry when gpsd loses the fix.\n"
-         "  -V                  Show version and exit\n",
+         "  -V                  Show version and exit\n\n\n\n"
+         "Note: gpxlogger sends error messages to the system log, not stderr."
+         "\n\n",
          progname,
 #ifdef HAVE_GETOPT_LONG
          export_default()->name,
 #endif
          export_default()->name);
+
     exit(EXIT_FAILURE);
 }
 
@@ -320,6 +326,7 @@ int main(int argc, char **argv)
     struct exportmethod_t *method = NULL;
     char   *file_in = NULL;
     const char *optstring = "?dD:e:f:F:ghi:lm:rV";
+    int err;
 #ifdef HAVE_GETOPT_LONG
     int option_index = 0;
     static struct option long_options[] = {
@@ -379,40 +386,25 @@ int main(int argc, char **argv)
             break;
        case 'f':       // Output file name.
             {
-                char   *fname = NULL;
-                time_t  t;
-                size_t  s = 0;
-                size_t fnamesize = strnlen(optarg, PATH_MAX) + 128;
+                char   fname[PATH_MAX];
+                time_t t = time(NULL);
 
-                t = time(NULL);
-                while (0 == s) {
-                    char *newfname = realloc(fname, fnamesize);
-
-                    if (NULL == newfname) {
-                        syslog(LOG_ERR, "realloc failed.");
-                        goto bailout;
-                    } else {
-                        fname = newfname;
-                    }
-                    s = strftime(fname, fnamesize-1, optarg, localtime(&t));
-                    if (!s) {
-                        /* expanded filename did not fit in string, try 
-                         * a bigger string */
-                        fnamesize += 1024;
+                size_t s = strftime(fname, sizeof(fname) - 11, optarg,
+                                    localtime(&t));
+                fname[s] = '\0';      // paranoia
+                if (0 < s) {
+                    gpxlogfile = fopen(fname, "w");
+                    if (NULL == gpxlogfile) {
+                        syslog(LOG_ERR,
+                               "Failed to open %s: %s(%d), logging to stdout.",
+                               fname, strerror(errno), errno);
+                        gpxlogfile = stdout;
                     }
+                } else {
+                    syslog(LOG_ERR, "strftime() failed, logging to stdout.");
                 }
-                fname[s] = '\0';;
-                gpxlogfile = fopen(fname, "w");
-                if (NULL == gpxlogfile) {
-                    syslog(LOG_ERR,
-                           "Failed to open %s: %s, logging to stdout.",
-                           fname, strerror(errno));
-                    gpxlogfile = stdout;
-                }
-            bailout:
-                free(fname);
-                break;
             }
+            break;
         case 'F':       // input file name.
             file_in = strdup(optarg);
             break;
@@ -500,10 +492,11 @@ int main(int argc, char **argv)
 
     // syslog (LOG_INFO, "---------- STARTED ----------");
 
-    if (0 != gps_open(source.server, source.port, &gpsdata)) {
+    err = gps_open(source.server, source.port, &gpsdata);
+    if (0 != err) {
         (void)fprintf(stderr,
-                      "%s: no gpsd running or network error: %d, %s\n",
-                      progname, errno, gps_errstr(errno));
+                      "%s: gps_open() failed  %s(%d) errno %s(%d)\n",
+                      progname, gps_errstr(err), err, strerror(errno), errno);
         exit(EXIT_FAILURE);
     }
 
