@@ -31,12 +31,12 @@
 // NTRIP 1.0 caster responses.  Based on Icecast audio servers
 #define NTRIP_SOURCETABLE       "SOURCETABLE 200 OK\r\n"
 #define NTRIP_ENDSOURCETABLE    "ENDSOURCETABLE"
-#define NTRIP_ICY               "ICY 200 OK"
+#define NTRIP_ICY               "ICY 200 OK\r\n"
 
 // NTRIP 2.0 caster responses.  Based on HTTP 1.1
 #define NTRIP_SOURCETABLE2      "Content-Type: gnss/sourcetable\r\n"
 #define NTRIP_BODY              "\r\n\r\n"
-#define NTRIP_HTTP              "HTTP/1.1 200 OK"
+#define NTRIP_HTTP              "HTTP/1.1 200 OK\r\n"
 
 // sourcetable stuff
 #define NTRIP_CAS               "CAS;"
@@ -46,8 +46,31 @@
 #define NTRIP_QSC               "\";\""
 
 // HTTP 1.1
-#define NTRIP_UNAUTH            "401 Unauthorized"
+#define NTRIP_UNAUTH            "401 Unauthorized\r\n"
+#define NTRIP_CHUNKED           "Transfer-Encoding: chunked\r\n"
 
+// ntrip_state() -- stringify conn_state
+static const char *ntrip_state(unsigned state)
+{
+    // NTRIP conn_states.  See include/gpsd.h
+    const char *ntrip_states[] = {
+        "INIT",
+        "SENT_PROBE",
+        "SENT_GET",
+        "ESTABLISHED",
+        "ERR",
+        "CLOSED",
+        "INPROGRESS",
+        "UNKNOWN",
+    };
+    unsigned num_states = sizeof(ntrip_states)/sizeof(ntrip_states[0]);
+
+    if (num_states <= state) {
+        // huh?
+        state = num_states - 1;
+    }
+    return ntrip_states[state];
+}
 
 // table to convert format string to enum ntrip_fmt
 static struct ntrip_fmt_s {
@@ -70,6 +93,7 @@ static struct ntrip_fmt_s {
     {"RTCM 3.1", FMT_RTCM3_1},
     {"RTCM3.1", FMT_RTCM3_1},
     {"RTCM 3.2", FMT_RTCM3_2},
+    {"RTCM3.2", FMT_RTCM3_2},       // for http://sapos.geonord-od.de:2101/
     {"RTCM32", FMT_RTCM3_2},
     {"RTCM 3.3", FMT_RTCM3_3},
     {"RTCM 3", FMT_RTCM3_0},
@@ -170,7 +194,7 @@ static void ntrip_str_parse(char *str, size_t len,
     if (NULL != (s = ntrip_field_iterate(NULL, s, eol, errout))) {
         hold->longitude = safe_atof(s);
     }
-    // <nmea>
+    // <nmea> 0 == do not send GGA, 1 == send GGA
     if (NULL != (s = ntrip_field_iterate(NULL, s, eol, errout))) {
         hold->nmea = atoi(s);
     }
@@ -190,6 +214,10 @@ static void ntrip_str_parse(char *str, size_t len,
             GPSD_LOG(LOG_WARN, errout,
                      "NTRIP: Got unknown {compress,encrypt}ion '%s'\n", s);
         }
+    } else {
+        GPSD_LOG(LOG_WARN, errout,
+                 "NTRIP: STR missing encryption and authentication fields\n");
+        return;    // done
     }
     // <authentication>
     if (NULL != (s = ntrip_field_iterate(NULL, s, eol, errout))) {
@@ -204,17 +232,29 @@ static void ntrip_str_parse(char *str, size_t len,
             GPSD_LOG(LOG_WARN, errout,
                      "NTRIP: Got unknown authenticatiion '%s'\n", s);
         }
+    } else {
+        GPSD_LOG(LOG_WARN, errout,
+                 "NTRIP: STR missing authenticatiion field\n");
+        return;    // done
     }
     // <fee>
-    if (NULL != (s = ntrip_field_iterate(NULL, s, eol, errout))) {
-        hold->fee = atoi(s);
+    s = ntrip_field_iterate(NULL, s, eol, errout);
+    if (NULL == s) {
+        // done, no more
+        return;
     }
+    hold->fee = atoi(s);
+
     // <bitrate>
-    if (NULL != (s = ntrip_field_iterate(NULL, s, eol, errout))) {
-        hold->bitrate = atoi(s);
+    s = ntrip_field_iterate(NULL, s, eol, errout);
+    if (NULL == s) {
+        // done, no more
+        return;
     }
+    hold->bitrate = atoi(s);
+
     // ...<misc>
-    while (NULL != (s = ntrip_field_iterate(NULL, s, eol, errout)));
+    // we don't care about extra fields
 }
 
 /* Parse the sourcetable, looking for a match to requested stream.
@@ -229,19 +269,30 @@ static int ntrip_sourcetable_parse(struct gps_device_t *device)
     ssize_t llen, len = 0;
     char *line;
     char buf[BUFSIZ / 2];   // half of BUFSIZE, so we can GPSD_LOG() it
-    socket_t fd = device->gpsdata.gps_fd;
+    socket_t fd = (socket_t)device->gpsdata.gps_fd;
 
     for (;;) {
         ssize_t rlen;
+        long long buf_avail;
 
-        memset(&buf[len], 0, sizeof(buf) - (size_t)len);
+        if (0 > len) {
+            // Pacify Coverity 498046
+            len = 0;
+        }
+        buf_avail = sizeof(buf) - len;
+        if (0 > buf_avail) {
+            // Pacify Coverity 498046
+            buf_avail = 0;
+        }
+        memset(&buf[len], 0, buf_avail);
         errno = 0;         // paranoia
         // read, leave room for trailing NUL
-        rlen = read(fd, &buf[len], sizeof(buf) - (size_t)(1 + len));
+        rlen = read(fd, &buf[len], buf_avail - 1);
+        // cast for 32-bit ints
         GPSD_LOG(LOG_RAW, &device->context->errout,
-                 "NTRIP: on fd %d len %zd  tried %zd, got %zd\n", fd, len,
-                 sizeof(buf) - (size_t)(1 + len), rlen);
-        if (-1 == rlen) {
+                 "NTRIP: on fd %ld len %zd  tried %zd, got %zd\n",
+                 (long)fd, len, sizeof(buf) - (size_t)(1 + len), rlen);
+        if (0 > rlen) {
             if (EINTR == errno) {
                 continue;
             }
@@ -250,16 +301,18 @@ static int ntrip_sourcetable_parse(struct gps_device_t *device)
                 // not found a match, but there is no more data
                 return 0;
             }
+            // cast for 32-bit ints
             GPSD_LOG(LOG_ERROR, &device->context->errout,
-                     "NTRIP: stream read error %s(%d) on fd %d\n",
-                     strerror(errno), errno, fd);
+                     "NTRIP: stream read error %s(%d) on fd %ld\n",
+                     strerror(errno), errno, (long)fd);
             return -1;
         }
         if (0 == rlen) {     // server closed the connection
+            // cast for 32-bit ints
             GPSD_LOG(LOG_ERROR, &device->context->errout,
-                     "NTRIP: stream unexpected close %s(%d) on fd %d "
+                     "NTRIP: stream unexpected close %s(%d) on fd %ld "
                      "during sourcetable read\n",
-                     strerror(errno), errno, fd);
+                     strerror(errno), errno, (long)fd);
             return -2;
         }
 
@@ -268,7 +321,7 @@ static int ntrip_sourcetable_parse(struct gps_device_t *device)
         rlen = len;
         // line points to the next char in buf to analyze
         // rlen is length of all data in buf
-        // len is length of remaining data in buf, 
+        // len is length of remaining data in buf,
 
         GPSD_LOG(LOG_IO, &device->context->errout,
                  "NTRIP: source table buffer >%.*s<\n", (int)rlen, buf);
@@ -411,7 +464,7 @@ static int ntrip_sourcetable_parse(struct gps_device_t *device)
  *
  * Warning: Blocking.  if the host is unresponsive, this will hang forever.
  *
- * Return: file descriptor of connection
+ * Return: fntrip_stream_get_parseile descriptor of connection
  *         negative number on failure
  */
 static int ntrip_stream_req_probe(const struct ntrip_stream_t *stream,
@@ -475,8 +528,16 @@ static int ntrip_auth_encode(struct ntrip_stream_t *stream)
     memset(stream->authStr, 0, sizeof(stream->authStr));
     switch (stream->authentication) {
     case AUTH_NONE:
-        // nothing to do.
-        break;
+        if ('\0' == *stream->credentials) {
+            // nothing to do.
+            break;
+        }
+        /* The user provided a user:password, but the SOURCETABLE did
+         * not request AUTH_BASIC. The RTKLIB Ver 2.4.2 (2013) str2str
+         * forgets to ask for basic auth even when needed.
+         * So if the user gave us a u:p, send it anyway
+         */
+        FALLTHROUGH
     case AUTH_BASIC:
         // RFC 7617 Basic Access Authentication.
         // username may not contain a colon (")
@@ -570,19 +631,72 @@ static socket_t ntrip_stream_get_req(const struct ntrip_stream_t *stream,
     return dsock;
 }
 
-/* parse the stream header
+/* lexer_getline() -- get one line, ending in \n or \0, from lexer->inbuffer,
+ * put in lexer->outbuffer.  NUL terminate outbuffer.
+ *
+ * Assume: inbufptr is correct.  inbuffer is NUL terminated.
+ *
+ * Can not handle buffer wrap.
+ *
+ * Return: void
+ */
+static void lexer_getline(struct gps_lexer_t *lexer)
+{
+    unsigned i;
+
+    for (i = 0; i < sizeof(lexer->outbuffer) - 2; i++) {
+        unsigned char u;
+
+        if (0 == lexer->inbuflen ||
+            sizeof(lexer->inbuffer) <= lexer->inbuflen) {  // paranoia
+            // nothing left to read,  ending not found
+            break;
+        }
+        u = *lexer->inbufptr++;
+        lexer->outbuffer[i] = u;
+        lexer->inbuflen--;
+
+        if ('\0' == u) {
+            // found NUL
+            break;
+        }
+        if ('\n' == u) {
+            // found return
+            i++;
+            break;
+        }
+    }
+    lexer->outbuffer[i] = '\0';  // Ensure a NUL
+    lexer->outbuflen = i;
+}
+
+/* ntrip_stream_get_parse(s) -- read, then parse, the stream header.
+ * Assume the entire header is ready to be read, and is less than
+ * 1024 bytes.
+ *
  * Return: 0 == OK
  *         less than zero == failure
  */
-static int ntrip_stream_get_parse(const struct ntrip_stream_t *stream,
-                                  const int dsock,
-                                  const struct gpsd_errout_t *errout)
+static int ntrip_stream_get_parse(struct gps_device_t *device)
 {
-    char buf[BUFSIZ];
+    char dbgbuf[128];
     int opts;
+    const struct ntrip_stream_t *stream = &device->ntrip.stream;
+    // (int) to shut up cadacy about USE_QT.
+    const int dsock  = (int)device->gpsdata.gps_fd;
+    const struct gpsd_errout_t *errout = &device->context->errout;
+    ssize_t read_ret;         // value retuend from read()
+    struct gps_lexer_t *lexer = &device->lexer;
+    char *ibuf = (char *)lexer->inbuffer;
+    char *obuf = (char *)lexer->outbuffer;
+    bool got_header;
 
-    memset(buf, 0, sizeof(buf));
-    while (-1 == read(dsock, buf, sizeof(buf) - 1)) {
+    GPSD_LOG(LOG_PROG, errout,
+             "NTRIP: ntrip_stream_get_parse(fd %d)\n", dsock);
+    lexer_init(lexer, &device->context->errout);
+    /* We expect the header comes in as one TCP packet.
+     * dsock is still blocking, so get exactly 1024 bytes */
+    while (0 >= (read_ret = read(dsock, ibuf, 1024))) {
         if (EINTR == errno) {
             continue;
         }
@@ -591,35 +705,127 @@ static int ntrip_stream_get_parse(const struct ntrip_stream_t *stream,
                  strerror(errno), errno, dsock);
         return -1;
     }
-    buf[sizeof(buf) - 1] = '\0';   // pacify coverity about NUL-terminated.
+    ibuf[read_ret] = '\0';   // Make a nice NUL terminated string.
+    lexer->inbuflen = (size_t)read_ret;
+    lexer_getline(lexer);
+    GPSD_LOG(LOG_IO, errout,
+             "NTRIP: lexer_getline() >%s<\n",
+             gps_visibilize(dbgbuf, sizeof(dbgbuf),
+                            (char *)lexer->outbuffer, lexer->outbuflen));
 
-    // parse 401 Unauthorized
-    if (NULL != strstr(buf, NTRIP_UNAUTH)) {
+    /* check for which of the 4 things we expect to start the reply:
+     *
+     * 401 Unauthorized\r\n     -- missing or wrong authentication
+     * SOURCETABLE 200 OK\r\n   -- incorrect mount point requested
+     * ICY 200 OK\r\n           -- NTRIP v1
+     * HTTP/1.1 200 OK\r\n      -- NTRIP v2
+     *
+     * Anything else is not understood.
+    */
+
+    if (0 == strncmp(obuf, NTRIP_UNAUTH, sizeof(NTRIP_UNAUTH))) {
         GPSD_LOG(LOG_ERROR, errout,
                  "NTRIP: not authorized for %s\n", stream->url);
         return -1;
     }
-    // can't parse SOURCETABLE here
-    if (NULL != strstr(buf, NTRIP_SOURCETABLE)) {
-        GPSD_LOG(LOG_ERROR, errout,
-                 "NTRIP: caster doesn't recognize stream %s:%s/%s\n",
-                 stream->host, stream->port, stream->mountpoint);
-        return -1;
-    }
     // parse "ICY 200 OK" or "HTTP/1.1 200 OK"
-    if (NULL == strstr(buf, NTRIP_ICY) &&
-        NULL == strstr(buf, NTRIP_HTTP)) {
+    if (0 != strncmp(obuf, NTRIP_ICY, sizeof(NTRIP_ICY)) &&
+        0 != strncmp(obuf, NTRIP_HTTP, sizeof(NTRIP_HTTP))) {
         GPSD_LOG(LOG_ERROR, errout,
-                 "NTRIP: Unknown reply %s from caster: %s:%s/%s\n", buf,
+                 "NTRIP: Unknown reply %s from caster: %s:%s/%s\n", obuf,
                  stream->host, stream->port, stream->mountpoint);
         return -1;
     }
+
+    // first line is good.
+
+    /* The NTRIP v2.0 is heavily based on HTTP/1.1, with some casters
+     * also using chunked transfers, as defined by RFC 9112, chap. 7.1,
+     * Chunked Transfer Coding, like so:
+     *
+     *  HTTP/1.1 200 OK\r\n
+     *  [...headers...]\r\n
+     *  Transfer-Encoding: chunked\r\n
+     *  \r\n
+     *  64\r\n
+     *  x64-bytes-worth-of-binary-message
+     *  \r\n
+     *  27;\r\n
+     *  x27-bytes-worth-of-binary-message
+     *  \r\n
+     *  42;foo=bar\r\n
+     *  x42-bytes-worth-of-binary-message
+     *  \r\n
+     *
+     * Annoyingly the chunks are NOT aligned on NTRIP message boundaries.
+     * So one possible benefit is lost.
+     *
+     * http/2 removed support for chunking.  Good riddance!
+     *
+     */
+
+    got_header = false;
+    while (0 < lexer->inbuflen) {
+        lexer_getline(lexer);
+        GPSD_LOG(LOG_IO, errout,
+                 "NTRIP: lexer_getline() >%s<\n",
+                 gps_visibilize(dbgbuf, sizeof(dbgbuf),
+                                (char *)lexer->outbuffer, lexer->outbuflen));
+
+        // Chunking needed?
+        if (0 == strncmp(obuf, NTRIP_CHUNKED, sizeof(NTRIP_CHUNKED))) {
+            GPSD_LOG(LOG_PROG, errout,
+                     "NTRIP: caster sends chunked data\n");
+            lexer->chunked = true;
+        }
+        if ('\0' == *lexer->outbuffer) {
+            // done, never got end of headers.
+            break;
+        }
+        if (0 == strncmp(obuf, NTRIP_BR, sizeof(NTRIP_BR))) {
+            // done
+            got_header = true;
+            break;
+        }
+    }
+    if (false == got_header) {
+        GPSD_LOG(LOG_WARN, errout,
+                 "NTRIP: did not get end of headers.\n");
+        /* do something about it? If we are not chunked it'll work out
+         * anyway. */
+    }
+
     opts = fcntl(dsock, F_GETFL);
 
-    if (0 <= opts) {
+    if (-1 == opts) {
+        GPSD_LOG(LOG_ERROR, errout, "NTRIP: fcntl(%d) %s(%d)\n",
+                 dsock, strerror(errno), errno);
+    } else {
         (void)fcntl(dsock, F_SETFL, opts | O_NONBLOCK);
     }
-
+    // The excess data from this first read is now in device->lexer.
+    // So far we have only seen zero here.
+    GPSD_LOG(LOG_IO, errout,
+             "NTRIP: ntrip_stream_get_parse(), %zu leftover bytes\n",
+             lexer->inbuflen);
+    if (0 == lexer->inbuflen ||
+        sizeof(lexer->inbuffer) <= lexer->inbuflen) {  // paranoia
+        packet_reset(lexer);
+    } else {
+        /* The "leftover" is the start of the datastream. Chunked or
+         * unchunked. */
+        if (lexer->inbufptr != lexer->inbuffer) {
+            // Shift inbufptr to the start.  Yes, a bit brutal.
+            memmove(lexer->inbuffer, lexer->inbufptr, lexer->inbuflen);
+            lexer->inbufptr = lexer->inbuffer;
+        }
+        GPSD_LOG(LOG_IO, errout,
+                 "NTRIP: leftover: >%s<\n",
+                 gps_visibilize(dbgbuf, sizeof(dbgbuf),
+                                (char *)lexer->inbuffer, lexer->inbuflen));
+    }
+    // empty the outbuffer of the ehader stuff
+    lexer->inbufptr = lexer->inbuffer;
     return 0;
 }
 
@@ -806,35 +1012,39 @@ static int ntrip_reconnect(struct gps_device_t *device)
              device->gpsdata.dev.path);
     dsock = netlib_connectsock1(AF_UNSPEC, device->ntrip.stream.host,
                                 device->ntrip.stream.port,
-                                "tcp", SOCK_NONBLOCK, addrbuf, sizeof(addrbuf));
+                                "tcp", 1, false,
+                                addrbuf, sizeof(addrbuf));
     device->gpsdata.gps_fd = dsock;
     // nonblocking means we have the fd, but the connection is not
     // finished yet.  Connection may fail, later.
     if (0 > dsock) {
-        // no way to recover from this, except wait and try again later
+        /* no way to recover from this, except wait and try again later
+         * cast for 32-bit ints. */
         GPSD_LOG(LOG_ERROR, &device->context->errout,
-                 "NTRIP: ntrip_reconnect(%s) IP %s, failed: %s(%d)\n",
+                 "NTRIP: ntrip_reconnect(%s) IP %s, failed: %s(%ld)\n",
                  device->gpsdata.dev.path, addrbuf,
-                 netlib_errstr(dsock), dsock);
+                 netlib_errstr(dsock), (long)dsock);
         // set time for retry
         (void)clock_gettime(CLOCK_REALTIME, &device->ntrip.stream.stream_time);
         // leave in connextion closed state for later retry.
         device->ntrip.conn_state = NTRIP_CONN_CLOSED;
         return PLACEHOLDING_FD;
     }
-    // will have to wait for select() to confirm conenction, then send
-    // the ntrip request again.
+    /* will have to wait for select() to confirm conenction, then send
+     * the ntrip request again.
+     * cast for 32-bit ints */
     device->ntrip.conn_state = NTRIP_CONN_INPROGRESS;
     GPSD_LOG(LOG_PROG, &device->context->errout,
-             "NTRIP: ntrip_reconnect(%s) IP %s, fd %d NTRIP_CONN_INPROGRESS \n",
-             device->gpsdata.dev.path, addrbuf, dsock);
+             "NTRIP: ntrip_reconnect(%s) IP %s, fd %ld "
+             "NTRIP_CONN_INPROGRESS \n",
+             device->gpsdata.dev.path, addrbuf, (long)dsock);
 #else  // no SOCK_NONBLOCK
     GPSD_LOG(LOG_PROG, &device->context->errout,
              "NTRIP: ntrip_reconnect(%s) no SOCK_NONBLOCK, can't reconnect.\n",
              device->gpsdata.dev.path);
     device->gpsdata.gps_fd = -1;
 #endif  // no SOCK_NONBLOCK
-    return device->gpsdata.gps_fd;
+    return (int)device->gpsdata.gps_fd;
 }
 
 /* open a connection to a NTRIP broadcaster
@@ -843,16 +1053,18 @@ static int ntrip_reconnect(struct gps_device_t *device)
  * Return: 0 on success, or the new fd
  *         less than zero on failure
  */
-int ntrip_open(struct gps_device_t *device, char *orig)
+socket_t ntrip_open(struct gps_device_t *device, char *orig)
 {
     socket_t ret = -1;
     char buf[BUFSIZ];
     char outbuf[BUFSIZ];
     ssize_t blen;
 
+    // cast for 32-bit ints
     GPSD_LOG(LOG_PROG, &device->context->errout,
-             "NTRIP: ntrip_open(%s) fd %d state = %d\n",
-             orig, device->gpsdata.gps_fd,
+             "NTRIP: ntrip_open(%s) fd %ld state = %s(%d)\n",
+             orig, (long)device->gpsdata.gps_fd,
+             ntrip_state(device->ntrip.conn_state),
              device->ntrip.conn_state);
 
     switch (device->ntrip.conn_state) {
@@ -877,9 +1089,10 @@ int ntrip_open(struct gps_device_t *device, char *orig)
 
         ret = ntrip_stream_req_probe(&device->ntrip.stream,
                                      &device->context->errout);
+        // cast for 32-bit intptr_t
         GPSD_LOG(LOG_PROG, &device->context->errout,
-                 "NTRIP: ntrip_stream_req_probe(%s) ret %d\n",
-                 device->ntrip.stream.url, ret);
+                 "NTRIP: ntrip_stream_req_probe(%s) ret %ld\n",
+                 device->ntrip.stream.url, (long)ret);
         if (-1 == ret) {
             device->gpsdata.gps_fd = PLACEHOLDING_FD;
             device->ntrip.conn_state = NTRIP_CONN_ERR;
@@ -889,14 +1102,15 @@ int ntrip_open(struct gps_device_t *device, char *orig)
         // cant use device->lexer.pkt_time and gpsd_clear() reset it
         (void)clock_gettime(CLOCK_REALTIME, &device->ntrip.stream.stream_time);
 
-        device->gpsdata.gps_fd = ret;
+        device->gpsdata.gps_fd = (gps_fd_t)ret;
         device->ntrip.conn_state = NTRIP_CONN_SENT_PROBE;
         return ret;
     case NTRIP_CONN_SENT_PROBE:     // state = 1
         ret = ntrip_sourcetable_parse(device);
+        // cast for 32-bit intptr_t
         GPSD_LOG(LOG_PROG, &device->context->errout,
-                 "NTRIP: ntrip_sourcetable_parse(%s) = %d\n",
-                 device->ntrip.stream.mountpoint, ret);
+                 "NTRIP: ntrip_sourcetable_parse(%s) = %ld\n",
+                 device->ntrip.stream.mountpoint, (long)ret);
         if (0 > ret) {
             device->ntrip.conn_state = NTRIP_CONN_ERR;
             return -1;
@@ -905,6 +1119,9 @@ int ntrip_open(struct gps_device_t *device, char *orig)
             false == device->ntrip.stream.set) {
             return ret;
         }
+        if (NULL != device->gpsdata.update_fd) {
+            device->gpsdata.update_fd(device->gpsdata.gps_fd, false);
+        }
         (void)close(device->gpsdata.gps_fd);
         device->gpsdata.gps_fd = PLACEHOLDING_FD;
         GPSD_LOG(LOG_PROG, &device->context->errout,
@@ -932,12 +1149,13 @@ int ntrip_open(struct gps_device_t *device, char *orig)
             return -1;
         }
         device->gpsdata.gps_fd = ret;
+        if (NULL != device->gpsdata.update_fd) {
+            device->gpsdata.update_fd(device->gpsdata.gps_fd, true);
+        }
         device->ntrip.conn_state = NTRIP_CONN_SENT_GET;
         break;
     case NTRIP_CONN_SENT_GET:          // state = 2
-        ret = ntrip_stream_get_parse(&device->ntrip.stream,
-                                     device->gpsdata.gps_fd,
-                                     &device->context->errout);
+        ret = ntrip_stream_get_parse(device);
         if (-1 == ret) {
             (void)close(device->gpsdata.gps_fd);
             device->gpsdata.gps_fd = PLACEHOLDING_FD;
@@ -954,6 +1172,10 @@ int ntrip_open(struct gps_device_t *device, char *orig)
             break;
         }
         ret = ntrip_reconnect(device);
+        if (0 <= ret &&
+            NULL != device->gpsdata.update_fd) {
+            device->gpsdata.update_fd(ret, true);
+        }
         break;
     case NTRIP_CONN_INPROGRESS:      // state = 6
         // Need to send GET within about 40 seconds or caster tiems out.
@@ -982,19 +1204,20 @@ int ntrip_open(struct gps_device_t *device, char *orig)
                   gps_visibilize(outbuf, sizeof(outbuf), buf, blen));
 
         if (blen != write(device->gpsdata.gps_fd, buf, blen)) {
+            // cast for 32-bit intptr_t
             GPSD_LOG(LOG_ERROR, &device->context->errout,
-                     "NTRIP: stream write error %s(%d) on fd %d during "
+                     "NTRIP: stream write error %s(%d) on fd %ld during "
                      "get request\n",
-                     strerror(errno), errno, device->gpsdata.gps_fd);
+                     strerror(errno), errno, (long)device->gpsdata.gps_fd);
             device->ntrip.conn_state = NTRIP_CONN_ERR;
             // leave FD so deactivate_device() can remove from the
             // select() loop
         } else {
             GPSD_LOG(LOG_ERROR, &device->context->errout,
                      "NTRIP: stream write success get request\n");
-            device->ntrip.conn_state = NTRIP_CONN_ESTABLISHED;
+            device->ntrip.conn_state = NTRIP_CONN_SENT_GET;
         }
-        ret = device->gpsdata.gps_fd;
+        ret = (int)device->gpsdata.gps_fd;
         break;
     case NTRIP_CONN_ESTABLISHED:     // state = 3
         FALLTHROUGH
@@ -1004,43 +1227,59 @@ int ntrip_open(struct gps_device_t *device, char *orig)
     return ret;
 }
 
-// may be time to ship a usage report to the NTRIP caster
+// may be time to ship a GGA report to the NTRIP caster
 void ntrip_report(struct gps_context_t *context,
                   struct gps_device_t *gps,
                   struct gps_device_t *caster)
 {
-    static int count;
+    static int count = 0;
+    char buf[BUFSIZ];
+    ssize_t blen;
 
-    /*
-     * 10 is an arbitrary number, the point is to have gotten several good
+    if (0 == caster->ntrip.stream.nmea) {
+        return;   // no need to be here...
+    }
+    // cast for 32-bit intptr_t
+    GPSD_LOG(LOG_IO, &context->errout,
+             "NTRIP: = ntrip_report() fixcnt %d count %d caster %ld\n",
+             context->fixcnt, count, (long)caster->gpsdata.gps_fd);
+
+    /* 10 is an arbitrary number, the point is to have gotten several good
      * fixes before reporting usage to our NTRIP caster.
-     *
-     * count % 5 is as arbitrary a number as the fixcnt. But some delay
-     * was needed here
      */
-    count ++;
-    if (0 != caster->ntrip.stream.nmea &&
-        10 < context->fixcnt &&
-        0 == (count % 5)) {
-        if (-1 < caster->gpsdata.gps_fd) {
-            char buf[BUFSIZ];
-            ssize_t ret, blen;
+    if (10 > context->fixcnt) {
+        return;   // no good fix to send...
+    }
 
-            gpsd_position_fix_dump(gps, buf, sizeof(buf));
-            blen = strnlen(buf, sizeof(buf));
-            ret = write(caster->gpsdata.gps_fd, buf, blen);
-            if (blen == ret) {
-                GPSD_LOG(LOG_IO, &context->errout, "NTRIP: => caster %s\n",
-                         buf);
-            } else if (0 > ret) {
-                GPSD_LOG(LOG_ERROR, &context->errout,
-                         "NTRIP: ntrip_report() write(%d) error %s(%d)\n",
-                         caster->gpsdata.gps_fd, strerror(errno), errno);
-            } else {
-                GPSD_LOG(LOG_ERROR, &context->errout,
-                         "NTRIP: ntrip_report() short write(%d) = %zd\n",
-                         caster->gpsdata.gps_fd, ret);
-            }
+    /* count % 5 is as arbitrary a number as the fix dump delay.
+     * But some delay * was needed here
+     */
+    count++;
+    if (0 != (count % 5)) {
+        return;   // wait some more
+    }
+    if (0 > caster->gpsdata.gps_fd) {
+        return;   // huh?  No NTRIP fd to write to??
+    }
+
+    blen = gpsd_position_fix_dump(gps, buf, sizeof(buf));
+    if (0 < blen) {
+        ssize_t ret;
+
+        ret = write(caster->gpsdata.gps_fd, buf, blen);
+        if (blen == ret) {
+            GPSD_LOG(LOG_IO, &context->errout, "NTRIP: => caster %s\n",
+                     buf);
+        } else if (0 > ret) {
+            // cast for 32-bit intptr_t
+            GPSD_LOG(LOG_ERROR, &context->errout,
+                     "NTRIP: ntrip_report() write(%ld) error %s(%d)\n",
+                     (long)caster->gpsdata.gps_fd, strerror(errno), errno);
+        } else {
+            // cast for 32-bit intptr_t
+            GPSD_LOG(LOG_ERROR, &context->errout,
+                     "NTRIP: ntrip_report() short write(%ld) = %zd\n",
+                     (long)caster->gpsdata.gps_fd, ret);
         }
     }
 }
@@ -1050,23 +1289,26 @@ void ntrip_close(struct gps_device_t *session)
 {
     if (0 > session->gpsdata.gps_fd) {
         // UNALLOCATED_FD (-1) or PLACEHOLDING_FD (-2). Nothing to do.
+        // cast for 32-bit intptr_t
         GPSD_LOG(LOG_ERROR, &session->context->errout,
-                 "NTRIP: ntrip_close(%s), close(%d) bad fd\n",
-                 session->gpsdata.dev.path, session->gpsdata.gps_fd);
+                 "NTRIP: ntrip_close(%s), close(%ld) bad fd\n",
+                 session->gpsdata.dev.path, (long)session->gpsdata.gps_fd);
         session->gpsdata.gps_fd = PLACEHOLDING_FD;
         return;
     }
 
     if (-1 == close(session->gpsdata.gps_fd)) {
+        // cast for 32-bit intptr_t
         GPSD_LOG(LOG_ERROR, &session->context->errout,
-                 "NTRIP: ntrip_close(%s), close(%d), %s(%d)\n",
+                 "NTRIP: ntrip_close(%s), close(%ld), %s(%d)\n",
                  session->gpsdata.dev.path,
-                 session->gpsdata.gps_fd, strerror(errno), errno);
+                 (long)session->gpsdata.gps_fd, strerror(errno), errno);
     } else {
+        // cast for 32-bit intptr_t
         GPSD_LOG(LOG_IO, &session->context->errout,
-                 "NTRIP: ntrip_close(%s), close(%d)\n",
+                 "NTRIP: ntrip_close(%s), close(%ld)\n",
                  session->gpsdata.dev.path,
-                 session->gpsdata.gps_fd);
+                 (long)session->gpsdata.gps_fd);
     }
     // Prepare for a retry, don't use opentime as that gets reset elsewhere
     (void)clock_gettime(CLOCK_REALTIME, &session->ntrip.stream.stream_time);
