@@ -135,11 +135,11 @@ struct inner_context_t {
 
 #if defined(HAVE_SYS_TIMEPPS_H)
 static int get_edge_rfc2783(struct inner_context_t *,
-                         struct timespec *,
-                         int *,
-                         struct timespec *,
-                         int *,
-                         volatile struct timedelta_t *);
+                            struct timespec *,
+                            int *,
+                            struct timespec *,
+                            int *,
+                            volatile struct timedelta_t *);
 #endif  // defined(HAVE_SYS_TIMEPPS_H)
 
 static pthread_mutex_t ppslast_mutex = PTHREAD_MUTEX_INITIALIZER;
@@ -149,13 +149,26 @@ static pthread_mutex_t ppslast_mutex = PTHREAD_MUTEX_INITIALIZER;
  * This is needed to avoid warnings from some overly pedantic compilers.
  * Unfortunately, the return type is platform-dependent, making it
  * impractical to actually use the return value.
+ *
+ * glibc strerror_r() is not POSIX compliant if _GNU_SOURCE is defined!
+ * sometimes glibc strerror_r() returns a pointer to a static, and
+ * leaves buf untouched!
  */
-static void pps_strerror_r(int errnum, char *buf, size_t len)
+static const char *pps_strerror_r(int errnum, char *buf, size_t len)
 {
-    if (strerror_r(errnum, buf, len)) {
-        return;
+#ifdef STRERROR_R_STR
+    // glibc strerror_r() returns pointer to the requested string.
+    // which may, or may not, point to buf.
+    return strerror_r(errnum, buf, len);
+#else
+    // strerror_r() returns 0 on success
+    // assign to an integer to create a warning if strerror_r() returns char *
+    int ret = strerror_r(errnum, buf, len);
+    if (0 == ret) {
+        return buf;
     }
-    return;
+    return "strerror_r() failed";
+#endif
 }
 
 static void thread_lock(volatile struct pps_thread_t *pps_thread)
@@ -163,10 +176,10 @@ static void thread_lock(volatile struct pps_thread_t *pps_thread)
     int pthread_err = pthread_mutex_lock(&ppslast_mutex);
     if (0 != pthread_err) {
         char errbuf[BUFSIZ] = "unknown error";
-        pps_strerror_r(errno, errbuf, sizeof(errbuf));
         pps_thread->log_hook(pps_thread, THREAD_ERROR,
-                "PPS:%s pthread_mutex_lock() : %d:%s\n",
-                pps_thread->devicename, errno, errbuf);
+                "PPS:%s pthread_mutex_lock() : %s(%d)\n",
+                pps_thread->devicename,
+                pps_strerror_r(errno, errbuf, sizeof(errbuf)), errno);
     }
 }
 
@@ -175,10 +188,10 @@ static void thread_unlock(volatile struct pps_thread_t *pps_thread)
     int pthread_err = pthread_mutex_unlock(&ppslast_mutex);
     if (0 != pthread_err) {
         char errbuf[BUFSIZ] = "unknown error";
-        pps_strerror_r(errno, errbuf, sizeof(errbuf));
         pps_thread->log_hook(pps_thread, THREAD_ERROR,
-                    "TPPS:%s pthread_mutex_unlock(): %d:%s\n",
-                    pps_thread->devicename, errno, errbuf);
+                    "TPPS:%s pthread_mutex_unlock(): %s(%d)\n",
+                    pps_thread->devicename,
+                    pps_strerror_r(errno, errbuf, sizeof(errbuf)), errno);
     }
 }
 
@@ -188,10 +201,12 @@ static void thread_unlock(volatile struct pps_thread_t *pps_thread)
  * empty (NUL-terminated) string if failure */
 static void get_sysfs_var(const char *path, char *buf, size_t bufsize)
 {
-    buf[0] = '\0';
     int fd = open(path, O_RDONLY);
+
+    buf[0] = '\0';
     if (0 <= fd) {
         ssize_t r = read(fd, buf, bufsize - 1);
+
         if (0 < r) {
             buf[r - 1] = '\0';     // remove trailing \x0a
         } else {
@@ -214,15 +229,22 @@ int pps_check_fake(const char *name) {
 }
 
 // Get first "real" PPS device, skipping the fake, if any
-char *pps_get_first(void)
+const char *pps_get_first(void)
 {
-    if (pps_check_fake("pps0"))
+    if (pps_check_fake("pps0")) {
         return "/dev/pps1";
+    }
     return "/dev/pps0";
 }
 #endif    // __linux__
 
-// return handle for kernel pps, or -1; requires root privileges
+/* init_kernel_pps() initialize kernel PPS.
+ * sets KPPS file handle into inner_context->kernelpps_handle
+ * usually requires root privileges
+ *
+ * Return: 0 on sucess
+ *         less than zero on error
+ */
 static int init_kernel_pps(struct inner_context_t *inner_context)
 {
     pps_params_t pp;
@@ -247,9 +269,6 @@ static int init_kernel_pps(struct inner_context_t *inner_context)
      * Some Linuxes, like the RasPi's, have PPS devices preexisting.
      * Other OS have no way to automatically determine the proper /dev/ppsX.
      * Allow user to pass in an explicit PPS device path.
-     *
-     * (We use strncpy() here because this might be compiled where
-     * strlcpy() is not available.)
      */
     if (0 == strncmp(pps_thread->devicename, "/dev/pps", 8)) {
         if (pps_check_fake(pps_thread->devicename + 5)) {
@@ -258,7 +277,7 @@ static int init_kernel_pps(struct inner_context_t *inner_context)
                                  " timing will be inaccurate\n",
                                  pps_thread->devicename);
         }
-        (void)strncpy(path, pps_thread->devicename, sizeof(path)-1);
+        (void)strlcpy(path, pps_thread->devicename, sizeof(path) - 1);
     } else {
         char pps_num = '\0';  // /dev/pps[pps_num] is our device
         size_t i;             // to match type of globbuf.gl_pathc
@@ -270,13 +289,16 @@ static int init_kernel_pps(struct inner_context_t *inner_context)
 
         /* Attach the line PPS discipline, so no need to ldattach.
          * This activates the magic /dev/pps0 device.
-         * Note: this ioctl() requires root, and device is a tty */
+         * Note: this ioctl() requires root, and device is a tty.
+         * For testing, you can manually create /dev/ppsX:
+         *   stty -F /dev/ttyXX line 18
+         */
         if (0 > ioctl(pps_thread->devicefd, TIOCSETD, &ldisc)) {
             char errbuf[BUFSIZ] = "unknown error";
-            pps_strerror_r(errno, errbuf, sizeof(errbuf));
             pps_thread->log_hook(pps_thread, THREAD_INF,
-                "KPPS:%s cannot set PPS line discipline %d:%s\n",
-                pps_thread->devicename, errno, errbuf);
+                "KPPS:%s cannot set PPS line discipline %s(%d)\n",
+                pps_thread->devicename,
+                pps_strerror_r(errno, errbuf, sizeof(errbuf)), errno);
             return -1;
         }
 
@@ -344,31 +366,32 @@ static int init_kernel_pps(struct inner_context_t *inner_context)
     // Should be a valid descriptor by this point
     if (0 > ret) {
         char errbuf[BUFSIZ] = "unknown error";
-        pps_strerror_r(errno, errbuf, sizeof(errbuf));
+
         // sometimes geteuid() and geteiud() are long
         pps_thread->log_hook(pps_thread, THREAD_INF,
-                    "KPPS:%s running as %ld/%ld, cannot open %s: %d:%s\n",
+                    "KPPS:%s running as %ld/%ld, cannot open %s: %s(%d)\n",
                     pps_thread->devicename,
                     (long)getuid(), (long)geteuid(),
-                    path, errno, errbuf);
+                    path,
+                    pps_strerror_r(errno, errbuf, sizeof(errbuf)), errno);
         return -1;
     }
 
     pps_thread->log_hook(pps_thread, THREAD_INF,
-                "KPPS:%s RFC2783 path:%s, fd is %d\n",
-                pps_thread->devicename, path,
-                ret);
+                "KPPS:%s RFC2783 path:%s, fd %d\n",
+                pps_thread->devicename, path, ret);
 
-    /* RFC 2783 implies the time_pps_setcap() needs privileges *
+    /* RFC 2783 implies the time_pps_setcap() needs privileges
      * keep root a tad longer just in case */
     if (0 > time_pps_create(ret,
-                            (pps_handle_t *)&inner_context->kernelpps_handle)) {
+                    (pps_handle_t *)&inner_context->kernelpps_handle)) {
         char errbuf[BUFSIZ] = "unknown error";
-        pps_strerror_r(errno, errbuf, sizeof(errbuf));
+
         pps_thread->log_hook(pps_thread, THREAD_INF,
-                    "KPPS:%s time_pps_create(%d) failed: %d:%s\n",
+                    "KPPS:%s time_pps_create(%d) failed: %s(%d)\n",
                     pps_thread->devicename,
-                    ret, errno, errbuf);
+                    ret,
+                    pps_strerror_r(errno, errbuf, sizeof(errbuf)), errno);
         return -1;
     }
 
@@ -377,11 +400,12 @@ static int init_kernel_pps(struct inner_context_t *inner_context)
     if (0 > time_pps_getcap(inner_context->kernelpps_handle,
                             &inner_context->pps_caps)) {
         char errbuf[BUFSIZ] = "unknown error";
+
         inner_context->pps_caps = 0;
-        pps_strerror_r(errno, errbuf, sizeof(errbuf));
         pps_thread->log_hook(pps_thread, THREAD_INF,
-                    "KPPS:%s time_pps_getcap() failed: %d:%.100s\n",
-                    pps_thread->devicename, errno, errbuf);
+                    "KPPS:%s time_pps_getcap() failed: %s(%d)\n",
+                    pps_thread->devicename,
+                    pps_strerror_r(errno, errbuf, sizeof(errbuf)), errno);
         return -1;
     } else {
         pps_thread->log_hook(pps_thread, THREAD_INF,
@@ -403,9 +427,9 @@ static int init_kernel_pps(struct inner_context_t *inner_context)
     }
     if (0 != (PPS_CANWAIT & inner_context->pps_caps)) {
        // we can wait! so no need for TIOCMIWAIT
-       pps_thread->log_hook(pps_thread, THREAD_INF,
-                   "KPPS:%s have PPS_CANWAIT\n",
-                   pps_thread->devicename);
+        pps_thread->log_hook(pps_thread, THREAD_INF,
+                             "KPPS:%s have PPS_CANWAIT\n",
+                             pps_thread->devicename);
         inner_context->pps_canwait = true;
     }
 
@@ -436,11 +460,11 @@ static int init_kernel_pps(struct inner_context_t *inner_context)
 
     if (0 > time_pps_setparams(inner_context->kernelpps_handle, &pp)) {
         char errbuf[BUFSIZ] = "unknown error";
-        pps_strerror_r(errno, errbuf, sizeof(errbuf));
+
         pps_thread->log_hook(pps_thread, THREAD_ERROR,
-            "KPPS:%s time_pps_setparams(mode=0x%02X) failed: %d:%s\n",
+            "KPPS:%s time_pps_setparams(mode=0x%02X) failed: %s(%d)\n",
             pps_thread->devicename, pp.mode,
-            errno, errbuf);
+            pps_strerror_r(errno, errbuf, sizeof(errbuf)), errno);
         (void)time_pps_destroy(inner_context->kernelpps_handle);
         return -1;
     }
@@ -490,11 +514,12 @@ static int get_edge_tiocmiwait(volatile struct pps_thread_t *thread_context,
 
     if (0 != ioctl(thread_context->devicefd, TIOCMIWAIT, PPS_LINE_TIOC)) {
         char errbuf[BUFSIZ] = "unknown error";
-        pps_strerror_r(errno, errbuf, sizeof(errbuf));
+
         thread_context->log_hook(thread_context, THREAD_WARN,
-                "TPPS:%s ioctl(TIOCMIWAIT) failed: %d:%.40s\n",
-                thread_context->devicename, errno, errbuf);
-        return -1;;
+                "TPPS:%s ioctl(TIOCMIWAIT) failed: %s(%d)\n",
+                thread_context->devicename,
+                pps_strerror_r(errno, errbuf, sizeof(errbuf)), errno);
+        return -1;
     }
 
     /*
@@ -513,22 +538,24 @@ static int get_edge_tiocmiwait(volatile struct pps_thread_t *thread_context,
     if (0 > clock_gettime(CLOCK_REALTIME, clock_ts)) {
         // uh, oh, can not get time!
         char errbuf[BUFSIZ] = "unknown error";
-        pps_strerror_r(errno, errbuf, sizeof(errbuf));
+
         thread_context->log_hook(thread_context, THREAD_ERROR,
-                    "TPPS:%s clock_gettime() failed: %d:%.100s\n",
-                    thread_context->devicename, errno, errbuf);
-        return -1;;
+                    "TPPS:%s clock_gettime() failed: %s(%d)\n",
+                    thread_context->devicename,
+                    pps_strerror_r(errno, errbuf, sizeof(errbuf)), errno);
+        return -1;
     }
 
     /* got the edge, got the time just after the edge, now quickly
      * get the edge state */
     if (0 != ioctl(thread_context->devicefd, (unsigned long)TIOCMGET, state)) {
         char errbuf[BUFSIZ] = "unknown error";
-        pps_strerror_r(errno, errbuf, sizeof(errbuf));
+
+        // cast for 32-bit intptr_t
         thread_context->log_hook(thread_context, THREAD_ERROR,
-                    "TPPS:%s ioctl(%d, TIOCMGET) failed: %.100s(%d)\n",
-                    thread_context->devicename, thread_context->devicefd,
-                    errbuf, errno);
+                    "TPPS:%s ioctl(%ld, TIOCMGET) failed: %s(%d)\n",
+                    thread_context->devicename, (long)thread_context->devicefd,
+                    pps_strerror_r(errno, errbuf, sizeof(errbuf)), errno);
         return -1;
     }
     // end of time critical section
@@ -598,20 +625,22 @@ static int get_edge_rfc2783(struct inner_context_t *inner_context,
     memset((void *)&pi, 0, sizeof(pi));    // paranoia
     if (0 > time_pps_fetch(inner_context->kernelpps_handle, PPS_TSFMT_TSPEC,
                            &pi, &kernelpps_tv)) {
-
         char errbuf[BUFSIZ] = "unknown error";
-        pps_strerror_r(errno, errbuf, sizeof(errbuf));
-        if (ETIMEDOUT == errno || EINTR == errno) {
-                // just a timeout
-                thread_context->log_hook(thread_context, THREAD_INF,
-                                         "KPPS:%s kernel PPS timeout %d:%s\n",
-                                         thread_context->devicename,
-                                         errno, errbuf);
-                return 1;
+
+        if (ETIMEDOUT == errno ||
+            EINTR == errno) {
+            // just a timeout
+            thread_context->log_hook(thread_context, THREAD_INF,
+                "KPPS:%s kernel PPS timeout %s(%d)\n",
+                thread_context->devicename,
+                pps_strerror_r(errno, errbuf, sizeof(errbuf)), errno);
+            return 1;
         }
         thread_context->log_hook(thread_context, THREAD_WARN,
-                                 "KPPS:%s kernel PPS failed %s\n",
-                                 thread_context->devicename, errbuf);
+                                 "KPPS:%s kernel PPS failed %s(%d)\n",
+                                 thread_context->devicename,
+                                 pps_strerror_r(errno, errbuf,
+                                                sizeof(errbuf)), errno);
         return 0;
     }
     if (inner_context->pps_canwait) {
@@ -632,13 +661,13 @@ static int get_edge_rfc2783(struct inner_context_t *inner_context,
         *edge = 1;
         *prev_edge = 0;
         if (0 == pi.clear_timestamp.tv_sec) {
-                /* brain damaged pps-gpio sometimes never fills in clear
-                 * so make it look like an invisible pulse
-                 * if clear is the leading edge, then we are off by the
-                 * pulse width */
-                *prev_clock_ts = pi.assert_timestamp;
+            /* brain damaged pps-gpio sometimes never fills in clear
+             * so make it look like an invisible pulse
+             * if clear is the leading edge, then we are off by the
+             * pulse width */
+            *prev_clock_ts = pi.assert_timestamp;
         } else {
-                *prev_clock_ts = pi.clear_timestamp;
+            *prev_clock_ts = pi.clear_timestamp;
         }
         *clock_ts = pi.assert_timestamp;
     } else {
@@ -702,18 +731,18 @@ static void *gpsd_ppsmonitor(void *arg)
     int state_tio = 0;
     int state_last_tio = 0;
     struct timespec clock_ts_tio = {0, 0};
-    /* pulse stores the time of the last two edges */
+    // pulse stores the time of the last two edges
     struct timespec pulse_tio[2] = { {0, 0}, {0, 0} };
-#endif /* TIOCMIWAIT */
+#endif  // TIOCMIWAIT
 
 #if defined(HAVE_SYS_TIMEPPS_H)
     int64_t cycle_kpps = 0, duration_kpps = 0;
-    /* kpps_pulse stores the time of the last two edges */
+    // kpps_pulse stores the time of the last two edges
     struct timespec pulse_kpps[2] = { {0, 0}, {0, 0} };
-#endif /* defined(HAVE_SYS_TIMEPPS_H) */
+#endif  // defined(HAVE_SYS_TIMEPPS_H)
     bool not_a_tty = false;
 
-    /* Acknowledge that we've grabbed the inner_context data */
+    // Acknowledge that we've grabbed the inner_context data
     ((volatile struct inner_context_t *)arg)->pps_thread = NULL;
 
     /* before the loop, figure out how we can detect edges:
@@ -722,17 +751,19 @@ static void *gpsd_ppsmonitor(void *arg)
      * or if KPPS is deficient a combination of the two */
     if (0 > thread_context->devicefd ||
         0 == isatty(thread_context->devicefd)) {
+        // cast for 32-bit intptr_t
         thread_context->log_hook(thread_context, THREAD_PROG,
-            "KPPS:%s gps_fd:%d not a tty, can not use TIOMCIWAIT\n",
+            "KPPS:%s gps_fd:%ld not a tty, can not use TIOMCIWAIT\n",
             thread_context->devicename,
-            thread_context->devicefd);
+            (long)thread_context->devicefd);
         /* why do we care the device is a tty? so as not to ioctl(TIO..)
-        * /dev/pps0 is not a tty and we need to use it */
+         * /dev/pps0 is not a tty and we need to use it */
         not_a_tty = true;
     }
-    /* if no TIOCMIWAIT, we hope to have PPS_CANWAIT */
+    // if no TIOCMIWAIT, we hope to have PPS_CANWAIT
 
-    if (not_a_tty && !inner_context.pps_canwait) {
+    if (not_a_tty &&
+        !inner_context.pps_canwait) {
         /* for now, no way to wait for an edge, in the future maybe figure out
          * a sleep */
     }
@@ -782,13 +813,13 @@ static void *gpsd_ppsmonitor(void *arg)
      *       a few more sanity checks
      *       call the report hook with our PPS report
      */
-    while (thread_context->report_hook != NULL) {
+    while (NULL != thread_context->report_hook) {
         bool ok = false;
         char *log = NULL;
         char *edge_str = "";
 
         if (10 == ++unchanged) {
-            /* last ten edges no good, stop spinning, just wait 10 seconds */
+            // last ten edges no good, stop spinning, just wait 10 seconds
             unchanged = 0;
             thread_context->log_hook(thread_context, THREAD_WARN,
                     "PPS:%s unchanged state, ppsmonitor sleeps 10\n",
@@ -796,13 +827,14 @@ static void *gpsd_ppsmonitor(void *arg)
             (void)sleep(10);
         }
 
-        /* Stage One; wait for the next edge */
+        // Stage One; wait for the next edge
 #if defined(TIOCMIWAIT)
-        if (!not_a_tty && !inner_context.pps_canwait) {
+        if (!not_a_tty &&
+            !inner_context.pps_canwait) {
             int ret;
 
-            /* we are a tty, so can TIOCMIWAIT */
-            /* we have no PPS_CANWAIT, so must TIOCMIWAIT */
+            /* we are a tty, so can TIOCMIWAIT
+             * we have no PPS_CANWAIT, so must TIOCMIWAIT */
 
             ret = get_edge_tiocmiwait(thread_context, &clock_ts_tio,
                                       &state_tio, &last_fixtime);
@@ -823,16 +855,16 @@ static void *gpsd_ppsmonitor(void *arg)
              * edge - rising edge (1), falling edge (0) or invisible edge (0)
              */
 
-            /* calculate cycle and duration from previous edges */
+            // calculate cycle and duration from previous edges
             cycle_tio = timespec_diff_ns(clock_ts_tio, pulse_tio[edge_tio]);
-            cycle_tio /= 1000;  /* nsec to usec */
+            cycle_tio /= 1000;    // nsec to usec
             duration_tio = timespec_diff_ns(clock_ts_tio,
                                             pulse_tio[edge_tio ? 0 : 1]) / 1000;
 
-            /* save this edge so we know next cycle time */
+            // save this edge so we know next cycle time
             pulse_tio[edge_tio] = clock_ts_tio;
 
-            /* use this data */
+            // use this data
             ok = true;
             clock_ts = clock_ts_tio;
             state = edge_tio;
@@ -849,33 +881,33 @@ static void *gpsd_ppsmonitor(void *arg)
                     timespec_str(&clock_ts, ts_str1, sizeof(ts_str1)));
 
         }
-#endif /* TIOCMIWAIT */
+#endif  // TIOCMIWAIT
 
-        /* ok and log used by KPPS and TIOCMIWAIT */
+        // ok and log used by KPPS and TIOCMIWAIT
         log = NULL;
 #if defined(HAVE_SYS_TIMEPPS_H)
         if ((pps_handle_t)0 <= inner_context.kernelpps_handle) {
             int ret;
-            int edge_kpps = 0;       /* 0 = clear edge, 1 = assert edge */
-            /* time of the last edge */
+            int edge_kpps = 0;       // 0 = clear edge, 1 = assert edge
+            // time of the last edge */
             struct timespec clock_ts_kpps = {0, 0};
-            /* time of the edge before the last edge */
+            // time of the edge before the last edge
             struct timespec prev_clock_ts = {0, 0};
-            /* direction of next to last edge 1 = assert, 0 = clear */
+            // direction of next to last edge 1 = assert, 0 = clear
             int prev_edge = 0;
 
             /* get last and previous edges, in order
              * optionally wait for goood data
              */
             ret = get_edge_rfc2783(&inner_context,
-                         &prev_clock_ts,
-                         &prev_edge,
-                         &clock_ts_kpps,
-                         &edge_kpps,
-                         &last_fixtime);
+                                   &prev_clock_ts,
+                                   &prev_edge,
+                                   &clock_ts_kpps,
+                                   &edge_kpps,
+                                   &last_fixtime);
 
             if (-1 == ret) {
-                /* error, so break */
+                // error, so break
                 thread_context->log_hook(thread_context, THREAD_ERROR,
                             "PPS:%s die: RFC2783 Error\n",
                             thread_context->devicename);
@@ -883,26 +915,26 @@ static void *gpsd_ppsmonitor(void *arg)
             }
 
             if (1 == ret) {
-                /* no edge found, so continue */
-                /* maybe use TIOCMIWAIT edge instead?? */
+                /* no edge found, so continue
+                 * maybe use TIOCMIWAIT edge instead?? */
                 continue;
             }
             /* for now, as we have been doing all of gpsd 3.x, just
              *use the last edge, not the previous edge */
 
-            /* compute time from previous saved similar edge */
+            // compute time from previous saved similar edge
             cycle_kpps = timespec_diff_ns(clock_ts_kpps, pulse_kpps[edge_kpps]);
             cycle_kpps /= 1000;
-            /* compute time from previous saved dis-similar edge */
+            // compute time from previous saved dis-similar edge
             duration_kpps = timespec_diff_ns(clock_ts_kpps, prev_clock_ts) /
                             1000;
 
-            /* save for later */
+            // save for later
             pulse_kpps[edge_kpps] = clock_ts_kpps;
             pulse_kpps[edge_kpps ? 0 : 1] = prev_clock_ts;
-            /* sanity checks are later */
+            // sanity checks are later
 
-            /* use this data */
+            // use this data
             state = edge_kpps;
             edge = edge_kpps;
             edge_str = edge ? "Assert" : "Clear";
@@ -919,10 +951,11 @@ static void *gpsd_ppsmonitor(void *arg)
                 timespec_str(&clock_ts_kpps, ts_str1, sizeof(ts_str1)));
 
         }
-#endif /* defined(HAVE_SYS_TIMEPPS_H) */
+#endif // defined(HAVE_SYS_TIMEPPS_H)
 
-        if (not_a_tty && !inner_context.pps_canwait) {
-            /* uh, oh, no TIOMCIWAIT, nor RFC2783, die */
+        if (not_a_tty &&
+            !inner_context.pps_canwait) {
+            // uh, oh, no TIOMCIWAIT, nor RFC2783, die
             thread_context->log_hook(thread_context, THREAD_WARN,
                         "PPS:%s die: no TIOMCIWAIT, nor RFC2783 CANWAIT\n",
                         thread_context->devicename);
@@ -953,9 +986,12 @@ static void *gpsd_ppsmonitor(void *arg)
                         "PPS:%s %.10s pps-detect changed to %d\n",
                         thread_context->devicename, edge_str, state);
             unchanged = 0;
-        } else if ((180000 < cycle &&  220000 > cycle)      // 5Hz
-                || (900000 < cycle && 1100000 > cycle)      // 1Hz
-                || (1800000 < cycle && 2200000 > cycle)) {  // 0.5Hz
+        } else if ((180000 < cycle &&
+                    220000 > cycle) ||      // 5Hz
+                   (900000 < cycle &&
+                    1100000 > cycle) ||      // 1Hz
+                   (1800000 < cycle &&
+                    2200000 > cycle)) {     // 0.5Hz
 
             /* some pulses may be so short that state never changes
              * and some RFC2783 only can detect one edge */
@@ -966,7 +1002,7 @@ static void *gpsd_ppsmonitor(void *arg)
                         "PPS:%s %.10s pps-detect invisible pulse\n",
                         thread_context->devicename, edge_str);
         }
-        /* else, unchannged state, and weird cycle time */
+        // else, unchannged state, and weird cycle time
 
         state_last = state;
         // (long long) for 32-bit compat.  PRId64 segfaults
@@ -1023,10 +1059,10 @@ static void *gpsd_ppsmonitor(void *arg)
         } else if (201000 > cycle) {
             /* longer than 200 milliSec - 10%
              * shorter than 200 milliSec + 10%
-             * about 200 milliSec cycle */
-            /* looks like 5hz PPS pulse */
+             * about 200 milliSec cycle
+             * looks like 5hz PPS pulse */
             if (100000 > duration) {
-                /* this is the end of the long part */
+                // this is the end of the long part
                 /* BUG: how does the code know to tell ntpd
                  * which 1/5 of a second to use?? */
                 ok = true;
@@ -1034,16 +1070,16 @@ static void *gpsd_ppsmonitor(void *arg)
             }
         } else if (900000 > cycle) {
             /* longer than 200 milliSec + 10%
-             * shorter than 1.000 Sec - 10% */
-            /* Yes, 10% window.  The Raspberry Pi clock is very coarse
+             * shorter than 1.000 Sec - 10%
+             * Yes, 10% window.  The Raspberry Pi clock is very coarse
              * when it starts and chronyd may be doing a fast slew.
              * chronyd by default will slew up to 8.334% ! */
             log = "Too long for 5Hz, too short for 1Hz\n";
         } else if (1100000 > cycle) {
             /* longer than 1.000 Sec - 10%
-             * shorter than 1.000 Sec + 10% */
-            /* Yes, 10% window.  */
-            /* looks like 1Hz PPS pulse or square wave */
+             * shorter than 1.000 Sec + 10%
+             * Yes, 10% window.
+             * looks like 1Hz PPS pulse or square wave */
             if (0 == duration) {
                 ok = true;
                 log = "invisible pulse\n";
@@ -1120,32 +1156,33 @@ static void *gpsd_ppsmonitor(void *arg)
         /* FIXME, some GPS, like Skytraq, may output a the fixtime so
          * late in the cycle as to be ambiguous. */
         if (0 == last_fixtime.real.tv_sec) {
-            /* probably should log computed offset just for grins here */
+            // probably should log computed offset just for grins here
             ok = false;
             log = "missing last_fixtime\n";
-        } else if (ok && last_second_used >= last_fixtime.real.tv_sec) {
-            /* uh, oh, this second already handled */
+        } else if (ok &&
+                   last_second_used >= last_fixtime.real.tv_sec) {
+            // uh, oh, this second already handled
             ok = false;
             log = "this second already handled\n";
         }
 
         if (!ok) {
-            /* can not use this pulse, reject and retry */
+            // can not use this pulse, reject and retry
             thread_context->log_hook(thread_context, THREAD_PROG,
                         "PPS:%s %.10s ignored %.100s",
                         thread_context->devicename, edge_str,  log);
             continue;
         }
 
-        /* we have validated a goood cycle, mark it */
+        // we have validated a goood cycle, mark it
         unchanged = 0;
-        /* offset is the skew from expected to observed pulse time */
+        // offset is the skew from expected to observed pulse time
         struct timespec offset;
-        /* offset as a printable string */
+        // offset as a printable string
         char offset_str[TIMESPEC_LEN];
-        /* delay after last fix */
+        // delay after last fix
         struct timespec  delay;
-        /* delay as a printable string */
+        // delay as a printable string
         char delay_str[TIMESPEC_LEN];
         char *log1 = "";
         // ppstimes.real is the time we think the pulse represents
@@ -1153,8 +1190,8 @@ static void *gpsd_ppsmonitor(void *arg)
         struct timedelta_t ppstimes;
 
         thread_context->log_hook(thread_context, THREAD_RAW,
-                    "PPS:%s %.10s categorized %.100s",
-                    thread_context->devicename, edge_str, log);
+                                 "PPS:%s %.10s categorized %.100s",
+                                 thread_context->devicename, edge_str, log);
 
         /* FIXME! The GR-601W at 38,400 or faster can send the
          * serial fix before the interrupt event carrying the PPS
@@ -1172,7 +1209,7 @@ static void *gpsd_ppsmonitor(void *arg)
             // handle x.9999 as x+1
             ppstimes.real.tv_sec++;
         }
-        ppstimes.real.tv_nsec = 0;  /* need to be fixed for 5Hz */
+        ppstimes.real.tv_nsec = 0;  // need to be fixed for 5Hz
         ppstimes.clock = clock_ts;
 
         // Here would be a good place to apply qErr
@@ -1191,17 +1228,19 @@ static void *gpsd_ppsmonitor(void *arg)
          *       call the report hook with our PPS report
          */
 
-        if (0 > delay.tv_sec || 0 > delay.tv_nsec) {
+        if (0 > delay.tv_sec ||
+            0 > delay.tv_nsec) {
             thread_context->log_hook(thread_context, THREAD_RAW,
                         "PPS:%s %.10s system clock went backwards: %.20s\n",
                         thread_context->devicename,
                         edge_str,
                         delay_str);
             log1 = "system clock went backwards";
-        } else if ((2 < delay.tv_sec)
-          || (1 == delay.tv_sec && 100000000 < delay.tv_nsec)) {
-            /* system clock could be slewing so allow up to 1.1 sec delay */
-            /* chronyd can slew +/-8.33% */
+        } else if ((2 < delay.tv_sec) ||
+                   (1 == delay.tv_sec &&
+                    100000000 < delay.tv_nsec)) {
+            /* system clock could be slewing so allow up to 1.1 sec delay
+             * chronyd can slew +/-8.33% */
             thread_context->log_hook(thread_context, THREAD_RAW,
                         "PPS:%s %.10s no current GPS seconds: %.20s\n",
                         thread_context->devicename,
@@ -1234,13 +1273,14 @@ static void *gpsd_ppsmonitor(void *arg)
                 log1,
                 timespec_str(&clock_ts, ts_str1, sizeof(ts_str1)),
                 timespec_str(&offset, offset_str, sizeof(offset_str)));
-        /* end Stage four, end of the loop, do it again */
+        // end Stage four, end of the loop, do it again
     }
 #if defined(HAVE_SYS_TIMEPPS_H)
     if ((pps_handle_t)0 < inner_context.kernelpps_handle) {
         thread_context->log_hook(thread_context, THREAD_PROG,
-            "KPPS:%s descriptor cleaned up\n",
-            thread_context->devicename);
+            "KPPS:%s fd %d cleaned up\n",
+            thread_context->devicename,
+            inner_context.kernelpps_handle);
         (void)time_pps_destroy(inner_context.kernelpps_handle);
     }
 #endif
@@ -1259,7 +1299,7 @@ void pps_thread_activate(volatile struct pps_thread_t *pps_thread)
 {
     int retval;
     pthread_t pt;
-    struct timespec start_delay = {0, 1000000};  /* 1 ms */
+    struct timespec start_delay = {0, 1000000};  // 1 ms
     /*
      * FIXME: this launch code is not itself thread-safe!
      * It would be if inner_context could be auto, but the monitor
@@ -1267,11 +1307,11 @@ void pps_thread_activate(volatile struct pps_thread_t *pps_thread)
      * of this function would be guarded by a separate mutex.
      * Either that, or this should be an exception to the no-malloc rule.
      */
-    static struct inner_context_t       inner_context;
+    static struct inner_context_t inner_context;
 
     inner_context.pps_thread = pps_thread;
 #if defined(HAVE_SYS_TIMEPPS_H)
-    /* some operations in init_kernel_pps() require root privs */
+    // some operations in init_kernel_pps() require root privs
     (void)init_kernel_pps(&inner_context);
     if ((pps_handle_t)0 <= inner_context.kernelpps_handle) {
         pps_thread->log_hook(pps_thread, THREAD_INF,
@@ -1292,14 +1332,16 @@ void pps_thread_activate(volatile struct pps_thread_t *pps_thread)
     memset(&pt, 0, sizeof(pt));
     retval = pthread_create(&pt, NULL, gpsd_ppsmonitor, (void *)&inner_context);
     pps_thread->log_hook(pps_thread, THREAD_PROG, "PPS:%s thread %s\n",
-                pps_thread->devicename,
-                (retval==0) ? "launched" : "FAILED");
+                         pps_thread->devicename,
+                         (retval==0) ? "launched" : "FAILED");
     /* The monitor thread may not run immediately, particularly on a single-
      * core machine, so we need to wait for it to acknowledge its copying
      * of the inner_context struct before proceeding.
      */
-    while (inner_context.pps_thread)
-        (void) nanosleep(&start_delay, NULL);
+    while (inner_context.pps_thread) {
+        (void)nanosleep(&start_delay, NULL);
+
+    }
 }
 
 // cleanly terminate PPS thread
