@@ -1,6 +1,6 @@
 /* gpsutils.c -- code shared between low-level and high-level interfaces
  *
- * This file is Copyright 2010 by the GPSD project
+ * This file is Copyright by the GPSD project
  * SPDX-License-Identifier: BSD-2-clause
  */
 
@@ -32,6 +32,99 @@
 #include <QStringList>
 #endif
 
+// gnssId names
+const struct vlist_t vgnssId[] = {
+    {0, "GPS"},
+    {1, "SBAS"},
+    {2, "GAL"},
+    {3, "BDS"},
+    {4, "IMES"},
+    {5, "QZSS"},
+    {6, "GLO"},
+    {7, "NavIC"},
+    {0, NULL},
+};
+
+// mode val to mode string
+const struct vlist_t vmode_str[] = {
+    {1, "No Fix"},
+    {2, "2D Fix"},
+    {3, "3D Fix"},
+    {0, NULL},
+};
+
+// status val to status string
+const struct vlist_t vstatus_str[] = {
+    {0, "UNK"},
+    {1, "GPS"},
+    {2, "DGPS"},
+    {3, "RTK_FIX"},
+    {4, "RTK_FLT"},
+    {5, "DR"},
+    {6, "GNSSDR"},
+    {7, "TIME"},      // Surveyd
+    {8, "SIM "},
+    {0, NULL},
+};
+
+/* char2str(ch, vlist) - given a char, return a matching string.
+ *
+ * Return: pointer to string
+ */
+const char *char2str(unsigned char ch, const struct clist_t *clist)
+{
+    while (NULL != clist->str) {
+        if (clist->ch == ch) {
+            return clist->str;
+        }
+        clist++;
+    }
+    return "Unk";
+}
+
+/* flags2str(val, vlist) - given flags, return a matching string.
+ *
+ * flags the flags to find in vlist
+ * buffer -  the buffer to return string in
+ ( buflen - length of buffer
+ *
+ * Return: pointer to passed in buffer
+ *         A string matching the flags.
+ */
+const char *flags2str(unsigned long flags, const struct flist_t *flist,
+                      char *buffer, size_t buflen)
+{
+    buffer[0] = '\0';
+    while (NULL != flist->str) {
+        if (flist->val == (flist->mask & flags)) {
+            if ('\0' != buffer[0]) {
+                strlcat(buffer, ",", buflen);
+            }
+            strlcat(buffer, flist->str, buflen);
+        }
+        flist++;
+    }
+    return buffer;
+}
+
+/* val2str(val, vlist) - given a value, return a matching string.
+ *
+ * val the value to find in vlist
+ *
+ * Return: pointer to static string
+ *         The string matching val, or "Unk".
+ */
+const char *val2str(unsigned long val, const struct vlist_t *vlist)
+{
+    while (NULL != vlist->str) {
+        if (vlist->val == val) {
+            return vlist->str;
+        }
+        vlist++;
+    }
+    return "Unk";
+}
+
 /*
  * Berkeley implementation of strtod(), inlined to avoid locale problems
  * with the decimal point and stripped down to an atof()-equivalent.
@@ -310,6 +403,7 @@ void gps_clear_fix(struct gps_fix_t *fixp)
     fixp->geoid_sep = NAN;
     fixp->dgps_age = NAN;
     fixp->dgps_station = -1;
+    fixp->temp = NAN;
     fixp->wanglem = NAN;
     fixp->wangler = NAN;
     fixp->wanglet = NAN;
@@ -347,12 +441,29 @@ void gps_clear_att(struct attitude_t *attp)
     gps_clear_base(&attp->base);
 }
 
+// Clear a dop_t structure
 void gps_clear_dop( struct dop_t *dop)
 {
     dop->xdop = dop->ydop = dop->vdop = dop->tdop = dop->hdop = dop->pdop =
         dop->gdop = NAN;
 }
 
+// Clear a gst structure
+void gps_clear_gst( struct gst_t *gst)
+{
+    memset(&gst->utctime, 0, sizeof(gst->utctime));
+    gst-> rms_deviation =  NAN;
+    gst-> smajor_deviation =  NAN;
+    gst-> sminor_deviation =  NAN;
+    gst-> smajor_orientation =  NAN;
+    gst-> lat_err_deviation =  NAN;
+    gst-> lon_err_deviation =  NAN;
+    gst-> alt_err_deviation =  NAN;
+    gst-> ve_err_deviation =  NAN;
+    gst-> vn_err_deviation =  NAN;
+    gst-> vu_err_deviation =  NAN;
+}
+
 // stuff a log structure with recognizable out-of-band values
 void gps_clear_log(struct gps_log_t *logp)
 {
@@ -502,6 +613,12 @@ void gps_merge_fix(struct gps_fix_t *to,
         to->dgps_station = from->dgps_station;
     }
 
+    if (ANT_UNK != from->ant_stat) {
+        to->ant_stat = from->ant_stat;
+    }
+    if (0 < from->jam) {
+        to->jam = from->jam;
+    }
     // navdata stuff.  just wind angle and angle for now
     if (0 != (transfer & NAVDATA_SET)) {
         if (0 != isfinite(from->wanglem)) {
@@ -520,6 +637,9 @@ void gps_merge_fix(struct gps_fix_t *to,
             to->wspeedt = from->wspeedt;
         }
     }
+    if (0 != isfinite(from->temp)) {
+        to->temp = from->temp;
+    }
     if (0 != isfinite(from->wtemp)) {
         to->wtemp = from->wtemp;
     }
@@ -531,6 +651,8 @@ void gps_merge_fix(struct gps_fix_t *to,
  * mktime() takes struct tm as localtime.
  *
  * The inverse of gmtime(time_t)
+ *
+ * Return: -1 on error, set errno to EOVERFLOW
  */
 time_t mkgmtime(struct tm * t)
 {
@@ -539,6 +661,26 @@ time_t mkgmtime(struct tm * t)
     static const int cumdays[MONTHSPERYEAR] =
         { 0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334 };
 
+    // check ranges, ignore tm_isdst and max tm_year
+    if (0 > t->tm_sec ||
+        0 > t->tm_min ||
+        0 > t->tm_hour ||
+        1 > t->tm_mday ||
+        0 > t->tm_mon ||
+        0 > t->tm_year ||
+        0 > t->tm_wday ||
+        0 > t->tm_yday ||
+        61 < t->tm_sec ||
+        59 < t->tm_min ||
+        23 < t->tm_hour ||
+        31 < t->tm_mday ||
+        11 < t->tm_mon ||
+        6 < t->tm_wday ||
+        365 < t->tm_yday) {
+        errno = EOVERFLOW;
+        return -1;
+    }
+    errno = 0;
     year = 1900 + t->tm_year + t->tm_mon / MONTHSPERYEAR;
     result = (year - 1970) * 365 + cumdays[t->tm_mon % MONTHSPERYEAR];
     result += (year - 1968) / 4;
@@ -561,7 +703,7 @@ time_t mkgmtime(struct tm * t)
      * if (t->tm_isdst == 1)
      * result -= 3600;
      */
-    return (result);
+    return result;
 }
 
 // ISO8601 UTC to Unix timespec, no leapsecond correction.
@@ -570,133 +712,7 @@ timespec_t iso8601_to_timespec(const char *isotime)
     timespec_t ret;
 
 #ifndef __clang_analyzer__
-#ifndef USE_QT
-    double usec = 0;
-    struct tm tm;
-    memset(&tm,0,sizeof(tm));
-
-#ifdef HAVE_STRPTIME
-    {
-        char *dp = NULL;
-        dp = strptime(isotime, "%Y-%m-%dT%H:%M:%S", &tm);
-        if (NULL != dp &&
-            '.' == *dp) {
-            usec = strtod(dp, NULL);
-        }
-    }
-#else
-    /* Fallback for systems without strptime (i.e. Windows)
-     * This is a simplistic conversion for iso8601 strings only,
-     * rather than embedding a full copy of strptime() that handles
-     * all formats */
-    /* Thus avoiding needing to test for (broken) negative date/time
-     * numbers in token reading - only need to check the upper range */
-    bool failed = false;
-    char *isotime_tokenizer = strdup(isotime);
-    if (isotime_tokenizer) {
-        char *tmpbuf;
-        char *pch = strtok_r(isotime_tokenizer, "-T:", &tmpbuf);
-        int token_number = 0;
-
-        while (NULL != pch) {
-            double sec;
-            unsigned int tmp;
-
-            token_number++;
-            // Give up if encountered way too many tokens.
-            if (10 < token_number) {
-                failed = true;
-                break;
-            }
-            switch (token_number) {
-            case 1:  // Year token
-                tmp = atoi(pch);
-                if (9999 > tmp) {
-                    tm.tm_year = tmp - 1900; // Adjust to tm year
-                } else {
-                    failed = true;
-                }
-                break;
-            case 2: // Month token
-                tmp = atoi(pch);
-                if (13 > tmp) {
-                    tm.tm_mon = tmp - 1;  // Month indexing starts from zero
-                } else {
-                    failed = true;
-                }
-                break;
-            case 3: // Day token
-                tmp = atoi(pch);
-                if (32 > tmp) {
-                    tm.tm_mday = tmp;
-                } else {
-                    failed = true;
-                }
-                break;
-            case 4:  // Hour token
-                tmp = atoi(pch);
-                if (24 > tmp) {
-                    tm.tm_hour = tmp;
-                } else {
-                    failed = true;
-                }
-                break;
-            case 5:  // Minute token
-                tmp = atoi(pch);
-                if (60 > tmp) {
-                    tm.tm_min = tmp;
-                } else {
-                    failed = true;
-                }
-                break;
-            case 6:  // Seconds token
-                sec = safe_atof(pch);
-                // NB To handle timestamps with leap seconds
-                if (0 == isfinite(sec) &&
-                    0.0 <= sec &&
-                    61.5 > sec) {
-                    // Truncate to get integer value
-                    tm.tm_sec = (unsigned int)sec;
-                    // Get the fractional part (if any)
-                    usec = sec - (unsigned int)sec;
-                } else {
-                    failed = true;
-                }
-                break;
-            default:
-                break;
-            }
-            pch = strtok_r(NULL, "-T:", &tmpbuf);
-        }
-        free(isotime_tokenizer);
-        /* Split may result in more than 6 tokens if the TZ has any t's
-         * in it.  So check that we've seen enough tokens rather than
-         * an exact number */
-        if (6 > token_number) {
-              failed = true;
-        }
-    }
-    if (failed) {
-        memset(&tm,0,sizeof(tm));
-    } else {
-        /* When successful this normalizes tm so that tm_yday is set
-         * and thus tm is valid for use with other functions */
-        if ((time_t)-1 == mktime(&tm)) {
-            // Failed mktime - so reset the timestamp
-            memset(&tm,0,sizeof(tm));
-        }
-    }
-#endif
-    /*
-     * It would be nice if we could say mktime(&tm) - timezone + usec instead,
-     * but timezone is not available at all on some BSDs. Besides, when working
-     * with historical dates the value of timezone after an ordinary tzset(3)
-     * can be wrong; you have to do a redirect through the IANA historical
-     * timezone database to get it right.
-     */
-    ret.tv_sec = mkgmtime(&tm);
-    ret.tv_nsec = usec * 1e9;;
-#else
+#ifdef USE_QT
     double usec = 0;
 
     QString t(isotime);
@@ -706,8 +722,30 @@ timespec_t iso8601_to_timespec(const char *isotime)
         usec = sl[1].toInt() / pow(10., (double)sl[1].size());
     }
     ret.tv_sec = d.toTime_t();
-    ret.tv_nsec = usec * 1e9;;
-#endif
+    ret.tv_nsec = usec * 1e9;
+#else  // USE_QT
+    double usec = 0;
+    struct tm tm = {0};
+
+    {
+        char *dp = NULL;
+        dp = strptime(isotime, "%Y-%m-%dT%H:%M:%S", &tm);
+        if (NULL != dp &&
+            '.' == *dp) {
+            usec = strtod(dp, NULL);
+        }
+    }
+
+    /*
+     * It would be nice if we could say mktime(&tm) - timezone + usec instead,
+     * but timezone is not available at all on some BSDs. Besides, when working
+     * with historical dates the value of timezone after an ordinary tzset(3)
+     * can be wrong; you have to do a redirect through the IANA historical
+     * timezone database to get it right.
+     */
+    ret.tv_sec = mkgmtime(&tm);
+    ret.tv_nsec = usec * 1e9;
+#endif  // USE_QT
 #endif  // __clang_analyzer__
 
 #if 4 < SIZEOF_TIME_T
@@ -747,12 +785,7 @@ char *timespec_to_iso8601(timespec_t fixtime, char isotime[], size_t len)
     }
 #endif
 
-#ifdef HAVE_GMTIME_R
     (void)gmtime_r(&fixtime.tv_sec, &when);
-#else
-    // Fallback to try with gmtime_s - primarily for Windows
-    (void)gmtime_s(&when, &fixtime.tv_sec);
-#endif
 
     /*
      * Do not mess casually with the number of decimal digits in the
@@ -867,7 +900,10 @@ double earth_distance(double lat1, double lon1, double lat2, double lon2)
     return earth_distance_and_bearings(lat1, lon1, lat2, lon2, NULL, NULL);
 }
 
-// Wait for data until timeout, ignoring signals.
+/* Wait for data until timeout, ignoring signals.
+ *
+ * pselect() may set errno on error
+ */
 bool nanowait(int fd, struct timespec *to)
 {
     fd_set fdset;
@@ -875,6 +911,7 @@ bool nanowait(int fd, struct timespec *to)
     FD_ZERO(&fdset);
     FD_SET(fd, &fdset);
     TS_NORM(to);         // just in case
+    errno = 0;
     // sigmask is NULL, so equivalent to select()
     return pselect(fd + 1, &fdset, NULL, NULL, to, NULL) == 1;
 }
@@ -928,16 +965,18 @@ void datum_code_string(int code, char *buffer, size_t len)
 
 /* make up an NMEA 4.0 (extended) PRN based on gnssId:svId,
  * This does NOT match NMEA 4.10 and 4.11 where all PRN are 1-99,
- * except IMEA
+ * except IMES, QZSS, and some SBAS.
+ *
  * Ref Appendix A from u-blox ZED-F9P Interface Description
  * and
  * Section 1.5.3  M10-FW500_InterfaceDescription_UBX-20053845.pdf
  *
- * Return PRN, or zero for error
+ * Return PRN, less than one for error
+ *        -1 for GLONASS svid 255
  */
 short ubx2_to_prn(int gnssId, int svId)
 {
-    short nmea_PRN;
+    short nmea_PRN = 0;
 
     if (1 > svId) {
         // skip 0 svId
@@ -983,9 +1022,11 @@ short ubx2_to_prn(int gnssId, int svId)
         }
         break;
     case 3:
-        // BeiDou, ubx gnssid:svid 1..37 -> to 401-437
-        // BeiDou, ubx PRN         159..163,33..64 -> to 401-437 ??
-        if (37 >= svId) {
+        /* BeiDou, ubx gnssid:svid    1..37 -> to 401-437
+         * have seen 1-63 on F10 ProtVer 40.0, March 2025
+         *   ubx gnssid:svid    1..63 -> to 401-463
+         * BeiDou, ubx "single svid"  159..163,33..64 -> to 401-437 ?? */
+        if (63 >= svId) {
             nmea_PRN = svId + 400;
         } else if (159 > svId) {
             // skip bad svId
@@ -1036,8 +1077,10 @@ short ubx2_to_prn(int gnssId, int svId)
             return 0;
         } else if (96 >= svId) {
             nmea_PRN = svId;
-        } else {
+        } else if (255 == svId) {
             // skip bad svId, 255 == tracked, but unidentified, skip
+            nmea_PRN = -1;
+        } else {
             return 0;
         }
         break;
